#include "i3d/image3d.h"
#include "histogram.h"

using namespace i3d;

void ComputeHistogram(const Image3d<GRAY8>& img, int (&histogram)[256])
{

	//initiate histogram to zeros everywhere
	for (int i=0; i < 256; ++i)
		histogram[i]=0;

	//sweep the image and count how many times each pixel values was used
	const GRAY8 *p=img.GetFirstVoxelAddr();
	for (size_t i=0; i < img.GetImageSize(); ++i, ++p)
		++histogram[*p];
}


void ComputeHistogram(const i3d::Image3d<i3d::GRAY8>& img,
								const i3d::VOI<i3d::PIXELS> &voi,
								int (&histogram)[256])
{
	//initiate histogram to zeros everywhere
	for (int i=0; i < 256; ++i)
		histogram[i]=0;

	//working copy of the input voi as the result of the intersection
	//of the input voi with the input image
	//VOI<PIXELS> wVoi(0,0,0, img.GetSizeX(),img.GetSizeY(),img.GetSizeZ());
	VOI<PIXELS> wVoi(Vector3d<PIXELS::T_offset>(0,0,0), img.GetSize());
	wVoi *= voi;

	//sets the sweeping pointer at the beginning of the VOI
	const GRAY8 *p=img.GetVoxelAddr(wVoi.offset.x,wVoi.offset.y,wVoi.offset.z);

	//sweeps over the VOI
	for (size_t z=0; z < wVoi.size.z; ++z) {
		for (size_t y=0; y < wVoi.size.y; ++y) {
			for (size_t x=0; x < wVoi.size.x; ++x, ++p)
				++histogram[*p];

			//jumps within the image to the next line of the VOI
			p+=img.GetSizeX() - wVoi.size.x;
		}

		//jumps within the image to the next slice of the VOI
		p+=(img.GetSizeY() - wVoi.size.y) *img.GetSizeX();
	}
}
