#include <iostream>
#include "i3d/image3d.h"
#include "threshold.h"
#include "histogram.h"

using namespace std;
using namespace i3d;


//subtracts pixel-wise the value \e val with boundary under-flow check
void SubtractIntensity(Image3d<GRAY8> &img, const GRAY8 val)
{
	GRAY8 *p=img.GetFirstVoxelAddr();
	for (size_t i=0; i < img.GetImageSize(); ++i, ++p)
		*p=(*p < val)? 0 : *p-val;
}

void ProcessCells(void)
{
	//opens and reads GRAY 8bit input image
	Image3d<GRAY8> img;
	img.ReadImageICS("images/cell01-uneven_2D_GRAY8.ics");

	//sets up some voi
	VOI<PIXELS> voiA(140,180,0, 30,30,1); //ze svetle casti
	VOI<PIXELS> voiB(205,115,0, 30,30,1); //ze tmave casti
	VOI<PIXELS> voiC(250,210,0, 30,30,1); //ze tmave casti

	//zalozime si histogram
	int histogram[256];
	int threshold=0;

	//nyni, pro kazde VOI zvlast:
	ComputeHistogram(img,voiA,histogram);
	threshold=SuggestThreshold(histogram);
	cout << "(bright) Suggested threshold is " << threshold << endl;
	Threshold(img,voiA,threshold);

	ComputeHistogram(img,voiB,histogram);
	threshold=SuggestThreshold(histogram);
	cout << "(dark)   Suggested threshold is " << threshold << endl;
	Threshold(img,voiB,threshold);

	//jenom overit, zda se prahy budou lisit zhruba o 30
	SubtractIntensity(img,30);
	img.SaveImageICS("images/cell01-uneven_2D_GRAY8_subtracted30.ics");

	ComputeHistogram(img,voiC,histogram);
	threshold=SuggestThreshold(histogram);
	cout << "(dark)   Suggested threshold is " << threshold << endl;
	Threshold(img,voiC,threshold);

	img.SaveImageICS("images/cell01-uneven_2D_GRAY8_thresholded.ics");

	//jeste zkusime tmavou oblast se svetlym prahem
	//znovu nacist vstupni obrazek
	//img.ReadImageICS("images/cell01-uneven_2D_GRAY8.ics");
	//Threshold(img,voiA,53);
	//Threshold(img,voiB,53);
	//Threshold(img,voiC,53);
	//img.SaveImageICS("images/cell01-uneven_2D_GRAY8_thresholded_at53.ics");
}

void ProcessCellsBlockByBlock(void)
{
	//opens and reads GRAY 8bit input image
	Image3d<GRAY8> img;
	img.ReadImageICS("images/cell01-uneven_2D_GRAY8.ics");

	//sets up some voi
	VOI<PIXELS> voi(0,0,0, 60,60,30);

	//zalozime si histogram
	int histogram[256];
	int threshold=0;

	//SubtractIntensity(img,30);
	//img.SaveImageICS("images/cell01-uneven_2D_GRAY8_subtracted30.ics");

	for (voi.offset.z=0; voi.offset.z < (signed)img.GetSizeZ(); voi.offset.z+=voi.size.z)
		for (voi.offset.y=0; voi.offset.y < (signed)img.GetSizeY(); voi.offset.y+=voi.size.y)
			for (voi.offset.x=0; voi.offset.x < (signed)img.GetSizeX(); voi.offset.x+=voi.size.x)
			{
				ComputeHistogram(img,voi,histogram);
				threshold=SuggestThreshold(histogram);
				Threshold(img,voi,threshold);
			}

	img.SaveImageICS("images/cell01-uneven_2D_GRAY8_thresholded.ics");
}


int main(void) {
	try {
		//chytani vyjimek, umoznuje zobrazit chybova hlaseni knihoven i3dlibs
		//tato konstrukce rika, ze v tomto bloku {...} se maji chytat vyjimky

		//ProcessCells();
		ProcessCellsBlockByBlock();

		//pokud se program dostal az sem, nenastal zadny problem
		return(0);

	} //konec bloku "try" - chytani ruznych vyjimek
	catch (const char *e)
	{
		cout << e << endl;
	}
	catch (std::string &e)
	{
		cout << e << endl;
	}
	catch (i3d::IOException& e)
	{
		cout << e << endl;
	}
	catch (i3d::InternalException& e)
	{
		cout << e << endl;
	}
	catch (std::bad_alloc&)
	{
		cout << "Not enough memory." << endl;
	}
	catch (...)
	{
		cout << "System exception." << endl;
	}

	//pokud se program dostal sem, nastal predtim nejaky problem
	return(1);
}
