#include "i3d/image3d.h"
#include "threshold.h"

#include <iostream>

using namespace i3d;

void Threshold(Image3d<GRAY8> &img,const GRAY8 thres)
{
	GRAY8 *p=img.GetFirstVoxelAddr();
	for (size_t i=0; i < img.GetImageSize(); ++i, ++p)
	{
		*p=(*p > thres)? 100 : 0;
	}
}

void Threshold(i3d::Image3d<i3d::GRAY8> &img,
					const i3d::VOI<i3d::PIXELS> &voi,
					const i3d::GRAY8 thres)
{
	//working copy of the input voi as the result of the intersection
	//of the input voi with the input image
	//VOI<PIXELS> wVoi(0,0,0, img.GetSizeX(),img.GetSizeY(),img.GetSizeZ());
	VOI<PIXELS> wVoi(Vector3d<PIXELS::T_offset>(0,0,0), img.GetSize());
	wVoi *= voi;

	//sets the sweeping pointer at the beginning of the VOI
	GRAY8 *p=img.GetVoxelAddr(wVoi.offset.x,wVoi.offset.y,wVoi.offset.z);

	//sweeps over the VOI
	for (size_t z=0; z < wVoi.size.z; ++z) {
		for (size_t y=0; y < wVoi.size.y; ++y) {
			for (size_t x=0; x < wVoi.size.x; ++x, ++p) {
				*p=(*p > thres)? 100 : 0;
			}

			//jumps within the image to the next line of the VOI
			p+=img.GetSizeX() - wVoi.size.x;
		}

		//jumps within the image to the next slice of the VOI
		p+=(img.GetSizeY() - wVoi.size.y) *img.GetSizeX();
	}
}

int SuggestThreshold(const int (&histogram)[256])
{
	//find the maximum skipping a few left most (smallest) intensities
	int L=5;
	int max_val=0;

	for (int i=L; i < 256; ++i)
	{
		if (histogram[i] > max_val)
		{
			//we've found intensities that is even more often present in the image 
			//than what we have discovered so far
			max_val=histogram[i]; //remember the count
			L=i;					//remember for what intensity
		}
	}

	//find the greatest used intensity
	int R=255;
	while ((R > 0) && (histogram[R] == 0)) --R;

	if (R <= L)
	{
		std::cout << "discovered maximum at L=" << L
			<< " and max intensity R=" << R << "\n";
		std::cout << "analysis is probably going to fail...\n";
		R=L;
	}

	float tmp=0.15 * (float)(R-L);
	return ( L + (int)tmp );
}
